package com.mortals.xhx.module.certificate.service.impl;

import cn.hutool.core.text.StrSplitter;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.template.MetaTemplate;
import com.deepoove.poi.util.RegexUtils;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.common.code.ClassifyType;
import com.mortals.xhx.common.code.ComponentEnum;
import com.mortals.xhx.common.code.StatusEnum;
import com.mortals.xhx.common.formdesign.*;
import com.mortals.xhx.common.pdu.gen.component.ComponentCons;
import com.mortals.xhx.common.utils.ExcelUtil;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.certificate.dao.CertificateCatalogDao;
import com.mortals.xhx.module.certificate.model.*;
import com.mortals.xhx.module.certificate.service.CertificateCatalogService;
import com.mortals.xhx.module.certificate.service.CertificateChildService;
import com.mortals.xhx.module.certificate.service.CertificateClassifyService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * CertificateCatalogService
 * 证照目录 service实现
 *
 * @author zxfei
 * @date 2022-10-14
 */
@Service("certificateCatalogService")
public class CertificateCatalogServiceImpl extends AbstractCRUDServiceImpl<CertificateCatalogDao, CertificateCatalogEntity, Long> implements CertificateCatalogService {

    @Value("${upload.path}")
    private String filePath;

    @Value("${upload.url:http://192.168.0.98:17216/cpm/file/commonupload?prePath=/file/fileupload}")
    private String uploadUrl;

    @Value("${qrcode.url:http://rhpt.scjg.chengdu.gov.cn:8091/adap/spjy/process/generateQrCode.do?dto.applyId=}")
    private String qrcode;

    @Autowired
    private CertificateClassifyService certificateClassifyService;

    @Autowired
    private CertificateChildService certificateChildService;

    @Override
    protected void validData(CertificateCatalogEntity entity, Context context) throws AppException {
        if (StringUtils.isEmpty(entity.getCatalogName())) {
            throw new AppException("目录名称不能为空");
        }
        if (StringUtils.isEmpty(entity.getCatalogCode())) {
            throw new AppException("目录编码不能为空");
        }
        if (entity.getHolderType() == null) {
            throw new AppException("持有者类型不能为空");
        }
        if (entity.getClassifyId() == null) {
            throw new AppException("目录分类不能为空");
        }
        if (entity.getIndustryId() == null) {
            throw new AppException("所属行业不能为空");
        }
    }

    @Override
    protected void saveBefore(CertificateCatalogEntity entity, Context context) throws AppException {
        validData(entity, context);
        entity.setStatus(StatusEnum.ENABLE.getValue());
        if (StringUtils.isNotEmpty(entity.getTemplateFileUrl())) {
            //entity.setFormContent(parseDocxToJson(entity.getTemplateUrl()));
            parseDocxToJson(entity, false);
        }

    }

    @Override
    protected void updateBefore(CertificateCatalogEntity entity, Context context) throws AppException {
        validData(entity, context);
        CertificateCatalogEntity old = this.get(entity.getId());
        if (!old.getTemplateFileUrl().equals(entity.getTemplateFileUrl())) {
            if (StringUtils.isNotEmpty(entity.getTemplateFileUrl())) {
                //entity.setFormContent(parseDocxToJson(entity.getTemplateUrl()));
                parseDocxToJson(entity, false);
            }
        }
    }


    @Override
    protected CertificateCatalogEntity findBefore(CertificateCatalogEntity params, PageInfo pageInfo, Context context) throws AppException {
        if (StringUtils.isNotEmpty(params.getCatalogName())) {
            params.setCatalogName("%".concat(params.getCatalogName()).concat("%"));
        }
        return params;
    }

    @Override
    protected void findAfter(CertificateCatalogEntity params, PageInfo pageInfo, Context context, List<CertificateCatalogEntity> list) throws AppException {
        if (CollectionUtils.isNotEmpty(list)) {
            List<Long> classifIdList = list.parallelStream().map(CertificateCatalogEntity::getClassifyId).collect(Collectors.toList());
            CertificateClassifyQuery certificateClassifyQuery = new CertificateClassifyQuery();
            certificateClassifyQuery.setIdList(classifIdList);
            Map<Long, CertificateClassifyEntity> collect = certificateClassifyService.find(certificateClassifyQuery, context).parallelStream()
                    .collect(Collectors.toMap(CertificateClassifyEntity::getId, Function.identity(), (n1, n2) -> n1));

            for (CertificateCatalogEntity entity : list) {
                CertificateClassifyEntity certificateClassifyEntity = collect.get(entity.getClassifyId());
               // CertificateClassifyEntity certificateClassifyEntity = certificateClassifyService.get(entity.getClassifyId());
                if (certificateClassifyEntity != null) {
                    entity.setClassifyType(certificateClassifyEntity.getClassifyType());
                } else {
                    entity.setClassifyType(ClassifyType.PRINCIPAL.getValue());
                }
            }
        }
        fillSubData(list);
    }

    @Override
    protected void findAfter(CertificateCatalogEntity params, Context context, List<CertificateCatalogEntity> list) throws AppException {
        fillSubData(list);
        super.findAfter(params, context, list);
    }

    private void fillSubData(List<CertificateCatalogEntity> list) {
        List<Long> idList = list.stream().map(i -> i.getId()).collect(Collectors.toList());
        if(ObjectUtils.isEmpty(idList))return;
        CertificateChildQuery certificateChildQuery = new CertificateChildQuery();
        certificateChildQuery.setCatalogIdList(idList);
        Map<Long, List<CertificateChildEntity>> certificateChildListMap = certificateChildService.find(certificateChildQuery).parallelStream().collect(Collectors.groupingBy(CertificateChildEntity::getCatalogId));
        list.forEach(item -> item.setCertificateChildList(certificateChildListMap.get(item.getId())));
    }

    @Override
    protected void saveAfter(CertificateCatalogEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getCertificateChildList())) {
            entity.getCertificateChildList().stream().peek(item -> {
                item.setCatalogId(entity.getId());
                item.setCatalogName(entity.getCatalogName());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
            }).count();
            certificateChildService.save(entity.getCertificateChildList());
        }
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(CertificateCatalogEntity entity, Context context) throws AppException {
        if (!ObjectUtils.isEmpty(entity.getCertificateChildList())) {
            Long[] certificateChildIds = certificateChildService.find(new CertificateChildQuery().catalogId(entity.getId())).stream().map(CertificateChildEntity::getId).toArray(Long[]::new);
            certificateChildService.remove(certificateChildIds, context);
            entity.getCertificateChildList().stream().peek(item -> {
                item.setCatalogId(entity.getId());
                item.setCreateUserId(this.getContextUserId(context));
                item.setCreateTime(new Date());
                item.setUpdateUserId(this.getContextUserId(context));
                item.setUpdateTime(new Date());
            }).count();
            certificateChildService.save(entity.getCertificateChildList());
        }
        super.updateAfter(entity, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        List<CertificateChildEntity> certificateChildlist = certificateChildService.find(new CertificateChildQuery().catalogIdList(Arrays.asList(ids)));
        certificateChildService.removeList(certificateChildlist, context);
        super.removeAfter(ids, context, result);
    }

    private void parseDocxToJson(CertificateCatalogEntity entity, boolean onlyExcel) {
        String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
        String filepath = rootPath + entity.getTemplateFileUrl();
        ConfigureBuilder builder = Configure.builder();

        builder.setValidErrorHandler(new Configure.DiscardHandler());
        builder.useDefaultEL(false);
        builder.buildGrammerRegex(RegexUtils.createGeneral("{{", "}}"));
        //builder.buildGramer("[","]");
        XWPFTemplate template = XWPFTemplate.compile(filepath, builder.build());
        FormDesignRoot root = new FormDesignRoot();
        FormDesignConfig config = new FormDesignConfig();
        config.initAttribute();
        root.setConfig(config);
        List<ListItem> collect = new ArrayList<>();
        HashSet<String> hs = new HashSet<>(); //去除重复标签
        List<String> excelList = new ArrayList<>();      //Excel模板
//        excelList.add("证照目录ID");
//        excelList.add("证照编号");
//        excelList.add("证照名称");
//        excelList.add("颁发时间");
//        excelList.add("取件人姓名");
//        excelList.add("取件人证件号码");
//        excelList.add("持有者名称");
//        excelList.add("持有者证件号码");
//        excelList.add("企业名称");
        for (MetaTemplate item : template.getElementTemplates()) {
            if (hs.contains(item.variable())) {
                continue;
            } else {
                hs.add(item.variable());
            }
            //识别简单输入
            String val = StrUtil.strip(item.variable(), "{{", "}}");
            excelList.add(val);
            List<String> keys = StrSplitter.split(val, "_", true, true);
            //切分分隔符后动态 创建组件  i_1_姓名（第一个代表类型，第二个如果是多个累加，第三个显示label）
            if (keys.size() > 2) {
                String label = keys.get(2);
                String type = keys.get(0);
                ComponentCons cons = new ComponentCons();
                cons.setLabel(label);
                cons.setVal(val);
                cons.setEl(item.variable());
                cons.setUploadFileUrl(uploadUrl);

                if (type.equalsIgnoreCase(ComponentEnum.DYNAMIC_TABLE.getValue())) {
                    HashSet<String> dhs = new HashSet<>(); //去除重复标签
                    List<TableColItem> colList = new ArrayList<>();
                    //表格 结构化绑定数据
                    ConfigureBuilder builderTable = Configure.builder();
                    builderTable.setValidErrorHandler(new Configure.DiscardHandler());
                    builderTable.useDefaultEL(false);
                    builderTable.buildGramer("[", "]");
                    XWPFTemplate templateTable = XWPFTemplate.compile(filepath, builderTable.build());

                    for (MetaTemplate col : templateTable.getElementTemplates()) {
                        if (dhs.contains(col.variable())) {
                            continue;
                        } else {
                            dhs.add(col.variable());
                        }
                        String colVal = StrUtil.strip(col.variable(), "[", "]");
                        if (!StrUtil.startWith(colVal, label)) {
                            continue;
                        }
                        List<String> colKeys = StrSplitter.split(colVal, "_", true, true);
                        //提取当前table中的列元素
                        if (colKeys.size() > 3) {
                            String colLabel = colKeys.get(3);
                            String colType = colKeys.get(1);
                            TableColItem tableColItem = new TableColItem();
                            tableColItem.setLabel(colLabel);
                            tableColItem.setColType(colType);
                            tableColItem.setVal(colVal);
                            colList.add(tableColItem);
                            //System.out.println("列元素" + col.variable());
                        }
                    }
                    cons.setColList(colList);
                    DesignComponent designComponent = DesignComponent.createType(type);
                    ListItem listItem = designComponent.buildDefaultComponent(cons);
                    collect.add(listItem);
                    continue;
                }
                DesignComponent designComponent = DesignComponent.createType(type);
                ListItem listItem = designComponent.buildDefaultComponent(cons);
                if (type.equals(ComponentEnum.QRCODE.getValue())) {
                    listItem.setValue(qrcode);
                }
                collect.add(listItem);
            }
        }
        root.setList(collect);
        String fileName = "file/fileupload/excel/" + entity.getCatalogName() + "_批量导入模板.xls";
        String excelFilePath = rootPath + fileName;
        ExcelUtil.createCatalogTemplate(excelFilePath, excelList);
        entity.setExcelFile(fileName);
        if (!onlyExcel) {
            entity.setFormContent(JSON.toJSONString(root));
        }
    }


    @Override
    public void generateExcel(Long id, Context context) {
        CertificateCatalogEntity entity = dao.get(id);
        parseDocxToJson(entity, true);
        CertificateCatalogEntity updata = new CertificateCatalogEntity();
        updata.setId(entity.getId());
        updata.setExcelFile(entity.getExcelFile());
        updata.setUpdateTime(new Date());
        dao.update(updata);
    }
}